#!/usr/bin/env python

"""
Prettyprinted document tree parser.

Copyright (C) 2017, 2018, 2019, 2023 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.tree.pretty import Node

class PrettyParser:

    "A prettyprinted document tree parser."

    formats = ["pretty"]

    def __init__(self, metadata):
        self.metadata = metadata

    def parse(self, s):

        "Parse the tree structure representation in 's'."

        indent = 0
        branches = []

        for line in s.split("\n"):
            line = line.rstrip()
            if not line:
                continue

            new_indent = line.rfind(" ") + 1
            node = Node(line[new_indent:])

            # Establish a branch to add nodes to.

            if not branches:
                branches.append(node)
            else:
                # Note the current node as outermost branch.

                if new_indent > indent:
                    branches.append(node)
                else:
                    # Reduced indent involves obtaining an inner branch again.

                    while indent > new_indent:
                        del branches[-1]
                        indent -= 2

                    # Note the current node as outermost branch.

                    branches[-1] = node

                # Append the current node to the parent branch.

                branches[-2].append(node)

            indent = new_indent

        return branches[0]

parser = PrettyParser

# vim: tabstop=4 expandtab shiftwidth=4
